<?php

namespace Drupal\link_obfuscator\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\link\Plugin\Field\FieldWidget\LinkWidget;

/**
 * Plugin implementation of the 'link_obfuscator_field_widget' widget.
 *
 * @FieldWidget(
 *   id = "link_obfuscator_field_widget",
 *   module = "link_obfuscator",
 *   label = @Translation("Link obfuscator field widget"),
 *   field_types = {
 *     "link",
 *   },
 * )
 */
class LinkObfuscatorFieldWidget extends LinkWidget {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'obfuscate_default' => FALSE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    // Add each of the enabled attributes.
    $item = $items[$delta];
    $options = $item->get('options')->getValue();
    $attributes = [];
    if (isset($options) && !empty($options) && !empty($options['attributes']) && isset($options['attributes'])) {
      $attributes = $options['attributes'];
    }
    $field_name = $this->fieldDefinition->getName();

    $parents = $element['#field_parents'];
    $parents[] = $field_name;
    $selector = $root = array_shift($parents);

    if ($parents) {
      $selector = $root . '[' . implode('][', $parents) . ']';
    }
    $element['options']['attributes'] = [
      '#type' => 'details',
      '#title' => $this->t('Obfuscate'),
      '#tree' => TRUE,
      '#open' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="' . $selector . '[' . $delta . '][uri]"]' => ['filled' => TRUE],
        ],
      ],
      'obfuscate' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Obfuscate this link'),
        '#default_value' => $this->getSetting('obfuscate_default') || $attributes['obfuscate'] ? $attributes['obfuscate'] : 0,
      ],
    ];

    if ($this->getSetting('obfuscate_default')) {
      $element['options']['attributes']['obfuscate']['#default_value'] = 1;
    }
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $elements['obfuscate_default'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Obfuscate by default'),
      '#default_value' => $this->getSetting('obfuscate_default'),
      '#description' => $this->t('By checking this checkbox, the link will be obfuscated by default.'),
      '#weight' => 99,
    ];
    return $elements;
  }

}
